% ================================================================================= % 
% gen_fig1.m 
%
% Here we compute the optimal policy starting with the baseline case and
% then performing a sensitivity analysis.
%
% Chris Boehm 
% December 2013
% ================================================================================= % 

clear all

options = optimoptions('fmincon','Display','off');

global beta sig kappa altrho_re altrho_u altrho_mpi altrho_my ...
    sig_re sig_u sig_mpi sig_my alpha

% Model Parameters 

% Preferences

beta        = 0.99;     % discount factor
theta       = 2/3;      % price stickyness

sig         = 1;  %1      % coefficient of relative risk aversion
eta         = 1;  %1      % labor supply elasticity

kappa       = (1-theta)*(1-theta*beta)/theta*(sig + 1/eta); %0.44;%0.34;%   %macroeconomic rate of price adjustment

% Shock Processes

altrho_re   = 0.9;%0.9;     % persistence of the efficient rate shock
altrho_u    = 0.5;%0.5;      % persistence of the cost-push shock
altrho_my   = 0.95;%0.95;      % persistence of the measurement error in y
altrho_mpi  = 0.0; %0.0;     % persistence of the measurement error in pi

sig_re      = 0.22;%0.22;        % standard deviation of the natural rate shock
sig_u       = 0.43;%0.43;        % standard deviation of the cost-push shock
sig_my      = 0.66;%0.66;      % standard deviation of the measurement error in y
sig_mpi     = 0.50;%0.50;      % standard deviation of the measurement error in pi

altrho_re_base   = altrho_re;     % persistence of the efficient rate shock
altrho_u_base    = altrho_u;      % persistence of the cost-push shock
altrho_my_base   = altrho_my;      % persistence of the measurement error in y
altrho_mpi_base  = altrho_mpi;     % persistence of the measurement error in pi

sig_re_base      = sig_re;        % standard deviation of the natural rate shock
sig_u_base       = sig_u;        % standard deviation of the cost-push shock
sig_my_base      = sig_my;      % standard deviation of the measurement error in y
sig_mpi_base     = sig_mpi;      % standard deviation of the measurement error in pi

% Welfare criterion

alpha = 1;        % weight on inflation (the weight on the gap is unity)
  

save('params_baseline.mat')

%sig_re      = sqrt(sig_re_base^2/(1-altrho_re_base^2)*(1-altrho_re^2));
%sig_u       = sqrt(sig_u_base^2/(1-altrho_u_base^2)*(1-altrho_u^2));
%sig_mpi     = sqrt(sig_mpi_base^2/(1-altrho_mpi_base^2)*(1-altrho_mpi^2));
%sig_my      = sqrt(sig_my_base^2/(1-altrho_my_base^2)*(1-altrho_my^2));

% Starting values for Taylor rule

TRcoefs0(1) = 0.5; %phiy = 0.6062      % reaction on output gap
TRcoefs0(2) = 2.0; %phipi = 2.0121       % reaction on inflation


tol = 0.0000001;

load params_baseline.mat
A = [-(1-beta) -kappa ]; b = [-kappa-tol];             % linear inequality constraints ("A*x <= b")
%A = []; b = [];             % linear inequality constraints ("A*x <= b")
Aeq =  []; beq =  [];       % linear equality constraints ("A*x = b")
lb = [0 0]; ub = [];        % lower and upper bounds

coefs_base = fmincon(@pol_obj, TRcoefs0, A, b, Aeq, beq, lb, ub, [], options);
% disp('Baseline Output and Inflation coefficient')
% disp([coefs_base(1)*4 coefs_base(2)])


% Create figure 1

clear all

global beta sig kappa altrho_re altrho_u altrho_mpi altrho_my ...
    sig_re sig_u sig_mpi sig_my alpha

% General settings

% Starting values for Taylor rule

TRcoefs0(1) = 2.0; %phipi = 2.0121       % reaction on inflation
TRcoefs0(2) = 0.6; %phiy = 0.6062      % reaction on output gap

tol = 0.000001;

load params_baseline.mat
A = [-(1-beta) -kappa ]; b = [-kappa-tol];            % linear inequality constraints ("A*x <= b")
%A = []; b = [];             % linear inequality constraints ("A*x <= b")
Aeq =  []; beq =  [];       % linear equality constraints ("A*x = b")
lb = [0 0]; ub = [5 5];        % lower and upper bounds

% [phiy_base phipi_base]= knitrolink(@pol_obj, TRcoefs0, A, b, Aeq, beq, lb, ub, [], options,'knitrooptions.txt');
[phipi_base phiy_base]= fmincon(@pol_obj, TRcoefs0, A, b, Aeq, beq, lb, ub, [], options);

% Scenario 1: Changing sig_mpi =========================

load params_baseline.mat
A = [-(1-beta) -kappa ]; b = [-kappa-tol];             % linear inequality constraints ("A*x <= b")

sig_mpi_grid = (0.3:0.01:0.7)';

coefs_sig_mpi = zeros(size(sig_mpi_grid,1),2);

for k = 1:length(sig_mpi_grid);

    sig_mpi = sig_mpi_grid(k);

%     coefs_sig_mpi(k,:) = knitrolink(@pol_obj, TRcoefs0, A, b, Aeq, beq, lb, ub, [], options,'knitrooptions.txt');
    coefs_sig_mpi(k,:) = fmincon(@pol_obj, TRcoefs0, A, b, Aeq, beq, lb, ub, [], options);
end

% Scenario 2: Changing sig_my =========================

load params_baseline.mat
A = [-(1-beta) -kappa ]; b = [-kappa-tol];             % linear inequality constraints ("A*x <= b")

sig_my_grid = (0.3:0.01:0.8)';

coefs_sig_my = zeros(size(sig_my_grid,1),2);

for k = 1:length(sig_my_grid);

    sig_my = sig_my_grid(k);

%     coefs_sig_my(k,:) = knitrolink(@pol_obj, TRcoefs0, A, b, Aeq, beq, lb, ub, [], options,'knitrooptions.txt');
    coefs_sig_my(k,:) = fmincon(@pol_obj, TRcoefs0, A, b, Aeq, beq, lb, ub, [], options);
end

fig1 = figure(1);
set(0,'defaulttextInterpreter','latex')
% set(0,'defaulttextInterpreter','none')
% set(gca, 'FontName', 'Calibri')

subplot(1,2,1)
plot(sig_mpi_grid,4*coefs_sig_mpi(:,1),'k--','Linewidth',1.5)
hold on
plot(sig_mpi_grid,coefs_sig_mpi(:,2),'k-','Linewidth',1.5)
hold off
xlabel('Standard Deviation of Inflation Measurement Error', 'FontSize',11)
% xlabel('\rho_{r^e}')
axis([sig_mpi_grid(1) sig_mpi_grid(end) 0 3])
% title('Innovation to measurement error in inflation')
line([0 1],[0 0],'Color','black')
line([sig_mpi_base sig_mpi_base],[0 10],'Color','black','LineStyle',':')
% text(sig_mpi_base-0.005,-0.1,' \uparrow baseline','FontSize',10)

subplot(1,2,2)
plot(sig_my_grid,4*coefs_sig_my(:,1),'k--','Linewidth',1.5)
hold on
plot(sig_my_grid,coefs_sig_my(:,2),'k-','Linewidth',1.5)
hold off
xlabel('Standard Deviation of Output Measurement Error','FontSize',11)
% xlabel('\rho_{r^e}')
% legend({'$\phi_y^*$','$\phi_{\pi}^*$'},'Location','southwest','Interpreter','latex') 
axis([sig_my_grid(1) sig_my_grid(end) 0 3])
% title('Innovation to measurement error in the gap')
line([0 1],[0 0],'Color','black')
line([sig_my_base sig_my_base],[0 10],'Color','black','LineStyle',':')
% text(sig_my_base-0.005,-0.1,' \uparrow baseline','FontSize',10)
legend({'$\phi_y^*$','$\phi_{\pi}^*$'},'Location','southwest','Interpreter','latex','FontSize',12) 


 set(fig1, 'PaperUnits', 'inches');
 scale = 2.2;
 x_width=scale*4; y_width=scale*3/2;
 set(fig1, 'PaperPosition', [0 0 x_width y_width]); 
 print('output\figure_1','-dpng','-r300')
