% ================================================================================= % 
% opt_TR.m 
%
% This is the general model in the optimal Taylor rule paper. 
%
% Chris Boehm 
% February 2014
% ================================================================================= % 

clear all;
%close all; 
%clc;

global beta sig kappa altrho_re altrho_u altrho_mpi altrho_my ...
    sig_re sig_u sig_mpi sig_my alpha

options = optimoptions('fmincon','Display','off');

nimp 	= 20;		% periods to be solved. 
nplot   = 20;    	% periods to be plotted

% Model Parameters 

% Preferences

beta        = 0.99;     % discount factor
theta       = 2/3;      % price stickyness

sig         = 1;        % coefficient of relative risk aversion
eta         = 1;        % labor supply elasticity

kappa       = (1-theta)*(1-theta*beta)/theta*(sig + 1/eta);   %macroeconomic rate of price adjustment



% Shock Processes

altrho_re   = 0.9;%0.9;     % persistence of the efficient rate shock
altrho_u    = 0.5;%0.5;      % persistence of the cost-push shock
altrho_my   = 0.95;%0.95;      % persistence of the measurement error in y
altrho_mpi  = 0.0; %0.0;     % persistence of the measurement error in pi

sig_re      = 0.22;%0.22;        % standard deviation of the natural rate shock
sig_u       = 0.43;%0.43;        % standard deviation of the cost-push shock
sig_my      = 0.66;%0.66;      % standard deviation of the measurement error in y
sig_mpi     = 0.50;%0.50;      % standard deviation of the measurement error in pi

% Welfare criterion

alpha = 1;          % weight on inflation (the weight on the gap is unity)

%==========================================================================
% Here either compute optimal Taylor rules or select the coefficients
% manually

% Optimal coefficients

% Starting values for Taylor rule

TRcoefs0(1) = 0.5; %phiy = 0.6062      % reaction on output gap
TRcoefs0(2) = 2.0; %phipi = 2.0121       % reaction on inflation

tol = 0.000001;

A = [-(1-beta) -kappa]; b = [-kappa-tol];             % linear inequality constraints ("A*x <= b")
%A = []; b = [];             % linear inequality constraints ("A*x <= b")
Aeq =  []; beq =  [];       % linear equality constraints ("A*x = b")
lb = [0 0]; ub = [];        % lower and upper bounds
% options = optimset();

coefs_base = fmincon(@pol_obj, TRcoefs0, A, b, Aeq, beq, lb, ub, [], options);

phiy  = coefs_base(1);
phipi = coefs_base(2);

disp('Standard Taylor Rule (TR1)');

disp('Output and Inflation coefficient')

disp([coefs_base(1)*4 coefs_base(2)])

%  % Manual selection
%                % optimal policy for baseline calibration
% phiy  = 0.8849;%0.8849      % reaction on output gap
% phipi = 3.9136;%3.9136       % reaction on inflation



figcount = 1;

%******************************************************************
%**** Begin setup for constructing log-linear system coef matrix **
%******************************************************************


nlead = 1;  % Number of leads in system 
nlag  = 1;  % Number of lags in system 

xnum = 7+4; 	%  Number of variables in system (plus shocks) 
neq = xnum;	    %  Number of equations (same)

% =========================================================================

colzero = 0+nlag*xnum;      % Position counter for start of contemp. coefs 
                            % i.e. if there is one lag and 5 equations then the "first"
                            % contemporaneous variable is at position 6 (6 = 1*5+cpos)
collead = 0+nlag*xnum+xnum; % Position counter for start of lead coefs 
collag  = 0;                % Position counter for start of lag coefs  

% Position counters

pipos       = 1;
ypos        = 2;
ipos        = 3;
repos       = 4;
upos        = 5;
mpipos      = 6;
mypos       = 7;
e_repos     = 8;
e_upos      = 9;
e_mpipos    = 10;
e_mypos     = 11;

% Indicators for contemporanous coefs for each variable: 

pizero      = colzero + pipos;
yzero       = colzero + ypos;
izero       = colzero + ipos;
rezero      = colzero + repos;
uzero       = colzero + upos;
mpizero     = colzero + mpipos;
myzero      = colzero + mypos;
e_rezero    = colzero + e_repos;
e_uzero     = colzero + e_upos;
e_mpizero   = colzero + e_mpipos;
e_myzero    = colzero + e_mypos;

% Indicators for lead coefficients for each variable: 

pilead      = collead + pipos;
ylead       = collead + ypos;
ilead       = collead + ipos;
relead      = collead + repos;
ulead       = collead + upos;
mpilead     = collead + mpipos;
mylead      = collead + mypos;
e_relead    = collead + e_repos;
e_ulead     = collead + e_upos;
e_mpilead   = collead + e_mpipos;
e_mylead    = collead + e_mypos;

% Indicators for lag coefficients for each variable: 

pilag       = collag + pipos;
ylag        = collag + ypos;
ilag        = collag + ipos;
relag       = collag + repos;
ulag        = collag + upos;
mpilag      = collag + mpipos;
mylag       = collag + mypos;
e_relag     = collag + e_repos;
e_ulag      = collag + e_upos;
e_mpilag    = collag + e_mpipos;
e_mylag     = collag + e_mypos;


% NOTE: these are all vectors of position counters
% Now I have one vector with all of the leads, lags, etc in one column

% Determine number of coefficients per equation: 

ncoef = neq*(nlag+nlead+1);

cof = zeros(neq,ncoef);             % Coef matrix --- Each row is an equation

% ================================================================================= % 
% ================================================================================= % 
% Fill in the coefficient matrix (cof)
% ================================================================================= % 
% ================================================================================= % 

% ================================================================================= % 
% Equation (1) Shock process of the efficient rate
% ================================================================================= % 

  cof(1,rezero)   = -1;
  cof(1,relag)    = altrho_re;
  cof(1,e_rezero) = 1;
  
% ================================================================================= % 
% Equation (2) Shock process of the cost-push shock
% ================================================================================= % 

  cof(2,uzero)   = -1;
  cof(2,ulag)    = altrho_u;
  cof(2,e_uzero) = 1;
  
% ================================================================================= % 
% Equation (3) Shock process of measurement error in pi
% ================================================================================= % 

  cof(3,mpizero)   = -1;
  cof(3,mpilag)    = altrho_mpi;
  cof(3,e_mpizero) = 1;
  
% ================================================================================= % 
% Equation (4) Shock process of measurement error in y
% ================================================================================= % 

  cof(4,myzero)   = -1;
  cof(4,mylag)    = altrho_my;
  cof(4,e_myzero) = 1;

% ================================================================================= % 
% Equation (5) Phillips curve
% ================================================================================= % 

  cof(5,pizero)        = -1;
  cof(5,yzero)         = kappa;
  cof(5,pilead)        = beta;
  cof(5,uzero)         = 1;
     
% ================================================================================= % 
% Equation (6) Euler equation
% ================================================================================= % 

  cof(6,yzero)      = -1;
  cof(6,izero)      = -1/sig;  
  cof(6,pilead)     = 1/sig;
  cof(6,ylead)      = 1; 
  cof(6,rezero)     = 1/sig;  
    
% ================================================================================= % 
% Equation (7) Taylor rule
% ================================================================================= %     

  cof(7,izero)      = -1;
  cof(7,yzero)      = phiy; %/4
  cof(7,myzero)     = phiy; %/4
  cof(7,pizero)     = phipi;
  cof(7,mpizero)    = phipi; 
    
  
% ================================================================================= % 
% Equation (8),(9), (10) and (11) Shocks
% ================================================================================= % 

  cof(8,e_rezero)       = 1;
  
  cof(9,e_uzero)        = 1;
  
  cof(10,e_mpizero)     = 1;
  
  cof(11,e_myzero)      = 1;

%*****************************************************************
%*************Begin Solution Algorithm ***************************
%*****************************************************************
%                                                                 
%  Solve a linear perfect foresight model using the gauss eig     
%  function to find the invariant subspace associated with the big 
%  roots.  This procedure will fail if the companion matrix is     
%  defective and does not have a linearly independent set of       
%  eigenvectors associated with the big roots.                     
%                                                                  
%  Input arguments:                                                
%                                                                  
%    h         Structural coef matrix (neq,neq*(nlag+1+nlag)).     
%    neq       Number of equations.                                
%    nlag      Number of lags.                                                                  
%    condn     lag tolerance used as a condition number test       
%              by numeric_shift and reduced_form.                  
%    uprbnd    Inclusive upper bound for the modulus of roots      
%              allowed in the reduced form.                        
%                                                                  
%  Output arguments:                                               
%                                                                  
%    cofb      Reduced form coefficient matrix (neq,neq*nlag).     
%    scof      Observable Structure                                
%    amat      Companion form matrix                               
%    b         Contemporaneous coefficient matrix                  
%    Model satisfies:                                              
%                                                                  
%    z(t) = amat*z(t-1) + b*e(t)                                   
%                                                                  
%    where the first neq elements of z(t) are the contemporaneous  
%    values of the variables in the model                          
%    and e(t) is the shock vector of conformable dimension         
%                                                                  
%    rts       Roots returned by eig.                              
%    ia        Dimension of companion matrix (number of non-trivial
%              elements in rts).                                   
%    nexact    Number of exact shiftrights.                        
%    nnumeric  Number of numeric shiftrights.                      
%    lgroots   Number of roots greater in modulus than uprbnd.     
%    mcode     Return code: see function aimerr.                   
%*****************************************************************


% Use AIM procedure to solve model: 
uprbnd = 1+1e-8;    % Tolerance values for AIM program 
condn = 1e-8;

% ---------------------------------------------------------------------
% Run AIM
% ---------------------------------------------------------------------

[cofb,rts,ia,nex,nnum,lgrts,mcode] = ...
       aim_eig(cof,neq,nlag,nlead,condn,uprbnd);

scof = obstruct(cof,cofb,neq,nlag,nlead);

% need to calculate amat and b
% ===============================
s0 = scof(:,(neq*nlag+1):neq*(nlag+1)); 	%Contemp. coefs from obs. structure
amat=zeros(neq*nlag,neq*nlag);   		% Initialize A matrix 
bmat=cofb(1:neq,((nlag-1)*neq+1):nlag*neq);  	% Lag 1 coefficients 
i=2;
while i<=nlag;
  bmat=[bmat cofb(1:neq,((nlag-i)*neq+1):(nlag-i+1)*neq)]; % Lag i coefs 
  i=i+1;
end;
amat(1:neq,:)=bmat;  				% Coefs for equations 
if nlag>1;
 amat((length(cofb(:,1))+1):length(amat(:,1)),1:neq*(nlag-1))=eye(neq*(nlag-1));
end;
b = zeros(length(amat(:,1)),length(s0(1,:)));
b(1:length(s0(:,1)),1:length(s0(1,:))) = inv(s0);  % Store coefs 
%b=b(:,shockvec);

% ================================================================================= % 
% Impulse Response Function
%
% ================================================================================= % 
% Get VAR form

AVAR = amat;
BVAR = b; 

%Variance and Autocorrelations

om = zeros(neq,neq);
om(e_repos,e_repos)     = sig_re^2;
om(e_upos,e_upos)       = sig_u^2;
om(e_mpipos,e_mpipos)   = sig_mpi^2;
om(e_mypos,e_mypos)     = sig_my^2;

V = cvar(AVAR,BVAR,om);            % Analytical way of solving for the covariance matrix

%==========================================================================

% omb = BVAR*om*BVAR'; 
% 
% V = omb;
% 
% for n = 1:300
%    V = AVAR*V*AVAR' + omb; 
% end

Vpi = V(pipos,pipos);

Vy = V(ypos,ypos);

WELFARE = Vy + alpha*Vpi;

disp('Vy and Vpi')

disp([Vy Vpi])

disp('Welfare criterion')

disp(WELFARE)

% Impulse response functions

shockH = zeros(neq,1);                     % Shock vector 
shockH(e_repos,1) = 1;                      % Shock variable, size of shock 
impdat = impf(AVAR,BVAR,shockH,nimp,neq);  % Call impulse respons proc 
dat=(1:nplot)';                            % Date variable for plotting 

% colordef white
% orient landscape
% 
% 
%  figure(figcount)
%   
%  subplot(3,3,1)
%  plot(dat,impdat(1:nplot,repos),'k-', 'LineWidth', 1.5); 
%  title('Efficient rate shock')
%  
%  subplot(3,3,2)
%  plot(dat,impdat(1:nplot,upos),'k-', 'LineWidth', 1.5); 
%  title('Cost-push shock') 
% 
%  subplot(3,3,3)
%  plot(dat,impdat(1:nplot,mpipos),'k-', 'LineWidth', 1.5); 
%  title('Measurement error in \pi shock')
%  
%  subplot(3,3,4)
%  plot(dat,impdat(1:nplot,mypos),'k-', 'LineWidth', 1.5); 
%  title('Measurement error in y shock') 
%  
%  subplot(3,3,5)
%  plot(dat,impdat(1:nplot,ypos),'k-', 'LineWidth', 1.5); 
%  title('Output gap')
%  
%  subplot(3,3,6)
%  plot(dat,impdat(1:nplot,pipos),'k-', 'LineWidth', 1.5);
%  title('Inflation')
%  
%  subplot(3,3,7)
%  plot(dat,impdat(1:nplot,ipos),'k-', 'LineWidth', 1.5);
%  title('Nominal interest rate')
%  
 % More impulse response functions
 
shock_re = zeros(neq,1);                     % Shock vector 
shock_re(e_repos,1) = sig_re;                      % Shock variable, size of shock 
base_impdat_re = impf(AVAR,BVAR,shock_re,nimp,neq);  % Call impulse respons proc 

shock_u = zeros(neq,1);                     % Shock vector 
shock_u(e_upos,1) = sig_u;                      % Shock variable, size of shock 
base_impdat_u = impf(AVAR,BVAR,shock_u,nimp,neq);  % Call impulse respons proc

shock_my = zeros(neq,1);                     % Shock vector 
shock_my(e_mypos,1) = sig_my;                      % Shock variable, size of shock 
base_impdat_my = impf(AVAR,BVAR,shock_my,nimp,neq);  % Call impulse respons proc

shock_mpi = zeros(neq,1);                     % Shock vector 
shock_mpi(e_mpipos,1) = sig_mpi;                      % Shock variable, size of shock 
base_impdat_mpi = impf(AVAR,BVAR,shock_mpi,nimp,neq);  % Call impulse respons proc

dat=(1:nplot)';                            % Date variable for plotting 

base_irfs_re = [base_impdat_re(:,ypos),base_impdat_re(:,pipos),base_impdat_re(:,ipos)];
base_irfs_u = [base_impdat_u(:,ypos),base_impdat_u(:,pipos),base_impdat_u(:,ipos)];
base_irfs_my = [base_impdat_my(:,ypos),base_impdat_my(:,pipos),base_impdat_my(:,ipos)];
base_irfs_mpi = [base_impdat_mpi(:,ypos),base_impdat_mpi(:,pipos),base_impdat_mpi(:,ipos)];

save('base_irfs','dat','base_irfs_re','base_irfs_u','base_irfs_my','base_irfs_mpi')
 
 
  